## About  
Dr. Lydia Moreno is an associate professor of Computational Neuroscience at Riverbend University’s School of Biological Sciences. Her work sits at the intersection of machine learning, systems neuroscience, and bioinformatics, aiming to unravel how neural circuits encode and process complex sensory information. She received her Ph.D. in Computational Biology from Baywood Institute of Technology and completed postdoctoral training in high‐throughput imaging analysis at Northborough Research Center.

## Education  
- **Ph.D., Computational Biology**  
  Baywood Institute of Technology, 2014–2019  
  • Dissertation: “Deep Learning Models for Decoding Neural Population Activity”  
- **M.Sc., Bioinformatics**  
  Riverbend University, 2012–2014  
  • Thesis: “Statistical Methods for Single‐Cell RNA‐Seq Analysis”  
- **B.Sc., Computer Science**  
  Crestfield College of Engineering, 2008–2012  
  • Graduated magna cum laude; President, Data Science Club  

## Research Interests  
- **Neural Encoding & Decoding**  
  Developing interpretable models that translate spike‐train data into behavioral predictions.  
- **Machine Learning for Biomedical Imaging**  
  Applying convolutional and graph neural networks to magnetic resonance and two‐photon microscopy data.  
- **Single‐Cell Omics**  
  Designing scalable pipelines for integrating multi‐modal single‐cell genomics and spatial transcriptomics.  
- **Explainable AI in Neuroscience**  
  Building frameworks to probe decision boundaries and feature attributions in black‐box models.

## Selected Projects  
### 1. DeepVision: High‐Resolution Neural Activity Mapping  
**Role**: Lead Developer & Project Coordinator  
- Designed a PyTorch‐based pipeline to segment two‐photon images of mouse visual cortex.  
- Integrated temporal convolutional networks to predict stimulus orientation from calcium‐imaging traces.  
- Deployed a Dockerized microservice for real‐time data processing in collaboration with Cascade Imaging Lab.

### 2. NeuroGraph: Graph Neural Networks for Circuit Analysis  
**Role**: Principal Investigator  
- Constructed graph representations of reconstructed neuron meshes from electron‐microscopy.  
- Trained GNNs to classify synapse types and infer connectivity motifs.  
- Published an open‐source toolkit in the NeuroTools suite, adopted by 12 labs worldwide.

### 3. CellMap: Multi‐Modal Single‐Cell Atlas  
**Role**: Co‐Principal Investigator  
- Developed a Nextflow workflow combining scRNA‐Seq, ATAC‐Seq, and spatial FISH data.  
- Implemented Bayesian clustering to resolve cell‐type hierarchies in human cortical organoids.  
- Presented at the International Conference on Single‐Cell Genomics (ICSCG) 2023.

## Publications  
1. Moreno, L., & Patel, A. (2024). “Temporal Convolutional Networks Decode Visual Stimuli from Two‐Photon Signals.” *Journal of Computational Neuroscience*, 45(2), 123–137.  
2. Moreno, L., Zhang, W., & Singh, R. (2023). “Graph Embedding Techniques Reveal Synaptic Motifs in EM Datasets.” *Neuroinformatics*, 21(4), 299–312.  
3. Moreno, L. et al. (2022). “Integrative Single‐Cell Atlas of Human Cerebral Organoids.” *Cell Systems*, 9(1), 67–82.  
4. Chen, J., Moreno, L., & Russo, M. (2021). “Explainable AI Techniques in Neural Decoding.” *Frontiers in Neuroscience*, 15, 756843.  

## Blog  
### Latest Posts
- **March 2025**: [Building Interpretable Deep Learning Models for Neuroscience](#)  
- **January 2025**: [Bridging Machine Learning and Single-Cell Genomics](#)  
- **November 2024**: [Scaling Graph Neural Networks for Connectomics](#)

## Teaching & Mentorship  
- **Undergraduate Course**: “Introduction to Computational Neuroscience” (Spring 2025)  
- **Graduate Seminar**: “Advanced Machine Learning Methods in Biology” (Fall 2024)  
- **Mentorship**: Supervised 8 Ph.D. students and 12 Master’s theses on topics ranging from neural data analysis to bioinformatics pipelines.  

## Skills & Tools  
- **Languages**: Python, R, C++  
- **Frameworks**: PyTorch, TensorFlow, scikit‐learn, NetworkX  
- **Data Tools**: Nextflow, Snakemake, Docker, Kubernetes  
- **Analysis**: MATLAB, Jupyter, Seurat, Scanpy  
- **Visualization**: D3.js, Plotly, Matplotlib  

## Contact  
- **Email**: lydia.moreno@riverbend.edu  
- **Office**: Blakely Hall, Room 312, Riverbend University  
- **Twitter**: [@LydiaNeuroLab](https://twitter.com/LydiaNeuroLab)  
- **GitHub**: [github.com/lydia‐moreno](https://github.com/lydia‐moreno)  
